use esp_idf_hal::gpio::OutputPin;
use esp_idf_hal::ledc;
use esp_idf_hal::ledc::{config::TimerConfig, Channel, Timer};
use esp_idf_hal::peripherals::Peripherals;
use esp_idf_hal::prelude::*;
use esp_idf_sys::EspError;
use std::thread;
use std::time::Duration;

type LedcChannel<P: OutputPin, T: ledc::HwTimer, C: ledc::HwChannel> = Channel<C, T, Timer<T>, P>;

pub struct Beep<P: OutputPin, T: ledc::HwTimer, C: ledc::HwChannel> {
    state: bool,
    stopped: bool,
    channel: LedcChannel<P, T, C>,
}

impl<P: OutputPin, T: ledc::HwTimer, C: ledc::HwChannel> Beep<P, T, C> {
    pub fn new(pin: P, timer: T, channel: C) -> Result<Self, EspError> {
        return Ok(Beep {
            state: false,
            stopped: false,
            channel: Self::init_channel(pin, timer, channel)?,
        });
    }

    fn init_channel(pin: P, timer: T, channel: C) -> Result<LedcChannel<P, T, C>, EspError> {
        let config = TimerConfig::default().frequency(1.kHz().into());
        let timer = Timer::new(timer, &config)?;
        let mut channel = Channel::new(channel, timer, pin)?;
        let max_duty = channel.get_max_duty();
        channel.set_duty(max_duty * 3 / 4)?;
        return Ok(channel);
    }

    pub fn toggle(&mut self) -> Result<(), EspError> {
        self.state = !self.state;
        if self.state {
            self.channel.set_duty(50)
        } else {
            self.channel.set_duty(0)
        }
    }

    pub fn play(&mut self) {
        loop {
            if self.stopped {
                break;
            }
            self.toggle().unwrap();
            thread::sleep(Duration::from_millis(500));

            self.toggle().unwrap();
            thread::sleep(Duration::from_millis(1500));
        }
    }
}
